package net.wanji.utc.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import static net.wanji.utc.common.constant.Constants.*;

/**
 * 控制指令接口
 *
 * @author wuxiaokai
 * @date 2022/11/15 13:21:10
 */
@Slf4j
@RestController
@RequestMapping("/controlCommand")
@Api(value = "控制指令接口", description = "控制指令接口", tags = "控制指令接口")
public class ControlCommandController {

    @Autowired
    private ControlCommandService hkControlCommandService;

    /**
     * 信号机方案下发
     *
     * @param schemeSendVO 方案下发VO
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public <T> Result<T> schemeSend(@RequestBody SchemeSendVO schemeSendVO) throws Exception {
        return hkControlCommandService.schemeSend(schemeSendVO);
    }

    /**
     * 下发信号机计划
     *
     * @param planSendVO 计划下发VO
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public <T> Result<T> planSend(@RequestBody PlanSendVO planSendVO) throws Exception {
        return hkControlCommandService.planSend(planSendVO);
    }

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public <T> Result<T> scheduleSend(@RequestBody ScheduleSendVO scheduleSendVO) throws Exception {
        return hkControlCommandService.scheduleSend(scheduleSendVO);
    }

    /**
     * 相位配时下发-（下发相位参数）
     *
     * @return {@link Result}<{@link T}>
     * @throws InterruptedException 中断异常
     */
    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public <T> Result<T> phaseTimingSend(@RequestBody PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        return hkControlCommandService.phaseTimingSend(phaseTimingSendVO);
    }

    /**
     * 全红控制
     *
     * @param commandVO 控制指令VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public <T> Result<T> allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return hkControlCommandService.setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), ALL_RED_CONTROL);
    }

    /**
     * 黄闪控制
     *
     * @param commandVO 控制指令VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public <T> Result<T> yellowLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return hkControlCommandService.setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), YELLOW_LIGHT_CONTROL);
    }

    /**
     * 关灯控制
     *
     * @param commandVO 控制指令VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public <T> Result<T> closeLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return hkControlCommandService.setSignalControl(commandVO.getCrossCode(), commandVO.getCommand(), CLOSE_LIGHT_CONTROL);
    }

    /**
     * 步进控制
     *
     * @param code    信号id
     * @param command 命令
     * @param stepNum 一步num
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(@RequestParam String code,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) throws Exception {
        return hkControlCommandService.stepControl(code, command, stepNum);
    }

    /**
     * 恢复时间表
     *
     * @param code 信号id
     * @return {@link Result}<{@link T}>
     */
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public <T> Result<T> recoverSchedule(@RequestParam String code) throws Exception {
        return hkControlCommandService.recoverSchedule(code);
    }

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return {@link Result}<{@link T}>
     * @throws Exception 异常
     */
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public <T> Result<T> lockControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return hkControlCommandService.lockControl(commandVO);
    }

    /**
     * 相位差下发
     */
    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public <T> Result<T> phaseDiffSend() {
        return hkControlCommandService.phaseDiffSend();
    }
}
